#ifndef PROCOBJ_H
#define PROCOBJ_H

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#include "procobjwin.h"
#include "common.h"
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <pwd.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <utmp.h>
#include <pwd.h>
#include <termcap.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ctype.h>
#include <setjmp.h>

#ifndef HZ
#define HZ 100
#endif

#define EXEC_PAGESIZE	4096

#ifndef NGROUPS
#define NGROUPS		32
#endif

#ifndef NOGROUP
#define NOGROUP		(-1)
#endif

#define MAXHOSTNAMELEN	64	/* max length of hostname */

typedef struct _procinfo1 {
	int pid;
	char cmd[40];
	char state;
	
	int 
		ppid, 		/* pid of parent process */
		pgrp, 		/* process group id */
		session, 	/* session id */
		tty, 			/* full device number of controlling terminal */
		tpgid,		/* terminal process group id */
		priority, 	/* kernel scheduling priority */
		nice,			/* standard unix nice level of process */
		signal, 		/* mask of pending signals */
		blocked, 	/* mask of blocked signals */
		sigignore, 	/* mask of ignored signals */
		sigcatch;	/* mask of caught  signals */
		
	unsigned long 
		flags, 		/* kernel flags for the process */
		min_flt, 	/* number of minor page faults since process start */
		cmin_flt, 	/* cumulative min_flt of process and child processes */
		maj_flt, 	/* number of major page faults since process start */
		cmaj_flt,	/* cumulative maj_flt of process and child processes */
		timeout, 	/* ? */
		it_real_value,	/* ? */
		vsize, 		/* number of pages of virtual memory ... */
		rss, 			/* resident set size from /proc/#/stat */
		rss_rlim, 	/* resident set size ... ? */
		start_code, /* address of beginning of code segment */
		end_code, 	/* address of end of code segment */
		start_stack, /* address of the bottom of stack for the process */
		kstk_esp,	/* kernel stack pointer */
		kstk_eip,	/* kernel stack pointer */
		wchan;		/* address of kernel wait channel proc is sleeping in */
					
	long 
		utime, 	/* user-mode CPU time accumulated by process */
		stime, 	/* kernel-mode CPU time accumulated by process */
		cutime, 	/* cumulative utime of process and reaped children */
		cstime,	/* cumulative stime of process and reaped children */
		start_time; /* start time of process -- seconds since 1-1-70 */
	
} procinfo1;

typedef struct _procinfo2 {
    /* theese members come from /proc/#/statm */
	long 
		size, 	/* total # of pages of memory */
		resident, /* number of resident set (non-swapped) pages (4k) */
		share,	/* number of pages of shared (mmap'd) memory */
		trs, 		/* text resident set size */
		lrs, 		/* shared-lib resident set size */
		drs, 		/* data resident set size */
		dt;		/* dirty pages */
} procinfo2;

procinfo1 stat2proc(char *S);
procinfo2 statm2proc(char *s);

char** file2strvec(char* directory, char* what);
int file2str(char *, char *, char *, int);
char* print_strlist(char **strs, char* sep, int max);

dev_t name_to_dev(char* name);
char* dev_to_name(dev_t num);
static int dev_to_devtab(int);
int tty_to_dev(char *tty);
void  dev_to_tty(char *tty, int dev);
char* abbrev_of_tty(char *tty);

char *user_from_uid(int uid);

void *xcalloc(void *pointer, int size);
void *xmalloc(unsigned int size);
void *xrealloc(void *oldp, unsigned int size);

@class ProcsFrame;

@interface ProcObj : NSView
{
	ProcsFrame *myFrame;
	NSWindow *win;
	ProcObjWin *explWin;
	NSString *name;
	NSString *path;
	int type;
	int myId;
	int parentid;
	int childsid[MAXPROC];
	int nchilds;
	int xShift;
	BOOL isSelect;
	BOOL isselectable;
	BOOL intree;
	
	NSCell *icnCell, *usrIcnCell, *ttyIcnCell, *cmdIcnCell;
	NSCell *nameCell, *usrCell, *statCell, *ttyCell, *commCell;
	
	BOOL isTty, isCmd;

	int uid;
	procinfo1 pinf1;
	procinfo2 pinf2;
	   
	char
		user[10],	/* user name corresponding to owner of process */
    	cmd[40],		/* basename of executable file in call to exec(2) */
    	state,		/* single-char code for process state (S=sleeping) */
    	ttyc[5],		/* string representation of controlling tty device */
		**environ,	/* environment (/proc/#/environ) */
		**cmdline;	/* command line (/proc/#/cmdline) */

	unsigned pcpu;    /* %CPU usage (is not filled in by readproc!!!) */
}

- (id)initInProcsFrame:(ProcsFrame *)parent
			  	  withPath:(NSString *)apath;

- (void)getProcData;
- (void)addChildId:(int)anid;
- (void)resetNchilds;
- (void)setId:(int)anid;
- (void)setParentid:(int)anid;
- (void)setXShift:(int)shift;
- (void)setInTree;
- (int)pid;
- (int)ppid;
- (NSString *)ppidname;
- (NSString *)myName;
- (NSString *)user;
- (NSString *)state;
- (NSString *)tty;
- (int)parentid;
- (int)myId;
- (long)start_time;
- (long)utime;
- (long)stime;
- (long)cutime;
- (long)cstime;
- (long)size;
- (long)resident;
- (long)share;
- (unsigned long)vsize;
- (unsigned long)start_code;
- (unsigned long)end_code;
- (NSString *)cmdLine;
- (int)xShift;
- (BOOL)intree;
- (int)nchilds;
- (int)childsid:(int)index;
- (BOOL)isSelect;

- (void)Activate;
- (void)DeActivate;
- (void)Select;
- (void)Unselect;
- (void)openExplWin;
- (void)closeExplWin:(id)sender;

@end


#endif
