#include "procobj.h"
#include "procsframe.h"

// --------------- PROCS READ (FUNCTIONS) --------------------
procinfo1 stat2proc(char *S)
{
	procinfo1 pinf;
	char buff[256] = "", *tmp;
	
	sprintf(buff, "%s", S);
	tmp = strrchr(buff, ')');	
	*tmp = '\0';
	
	sscanf(buff, "%d (%s", &pinf.pid, pinf.cmd);
	sscanf(tmp + 2,
	   "%c %d %d %d %d %d %lu %lu %lu %lu %lu %ld %ld %ld %ld %d "
	   "%d %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %d %d %d %d %lu",
	   &pinf.state, &pinf.ppid, &pinf.pgrp, &pinf.session, &pinf.tty, &pinf.tpgid,
	   &pinf.flags, &pinf.min_flt, &pinf.cmin_flt, &pinf.maj_flt, &pinf.cmaj_flt,
	   &pinf.utime, &pinf.stime, &pinf.cutime, &pinf.cstime, &pinf.priority, &pinf.nice,
	   &pinf.timeout, &pinf.it_real_value, &pinf.start_time, &pinf.vsize, &pinf.rss,
	   &pinf.rss_rlim, &pinf.start_code, &pinf.end_code, &pinf.start_stack,
	   &pinf.kstk_esp, &pinf.kstk_eip, &pinf.signal, &pinf.blocked, &pinf.sigignore,
	   &pinf.sigcatch, &pinf.wchan);

	if(pinf.tty == 0)
		pinf.tty = -1;

	return pinf;
}

procinfo2 statm2proc(char *s)
{
	procinfo2 pinf;

	sscanf(s, "%ld %ld %ld %ld %ld %ld %ld",
	   		&pinf.size, &pinf.resident, &pinf.share,
	   			&pinf.trs, &pinf.lrs, &pinf.drs, &pinf.dt);
	return pinf;
}

char** file2strvec(char* directory, char* what) {
	char buf[2048];	/* read buf bytes at a time */
	char *p, *rbuf = 0, *endbuf, **q, **ret;
	int fd, tot = 0, n, c, end_of_file = 0;
	int align;

	sprintf(buf, "%s/%s", directory, what);
    if ( (fd = open(buf, O_RDONLY, 0) ) == -1 ) return NULL;

    /* read whole file into a memory buffer, allocating as we go */
    while ((n = read(fd, buf, sizeof buf - 1)) > 0) {
	if (n < sizeof buf - 1)
	    end_of_file = 1;
	if (n == 0 && rbuf == 0)
	    return NULL;	/* process died between our open and read */
	if (n < 0) {
	    if (rbuf)
		free(rbuf);
	    return NULL;	/* read error */
	}
	if (end_of_file && buf[n-1])		/* last read char not null */
	    buf[n++] = '\0';			/* so append null-terminator */
	rbuf = xrealloc(rbuf, tot + n);		/* allocate more memory */
	memcpy(rbuf + tot, buf, n);		/* copy buffer into it */
	tot += n;				/* increment total byte ctr */
	if (end_of_file)
	    break;
    }
    close(fd);
    if (n <= 0 && !end_of_file) {
	if (rbuf) free(rbuf);
	return NULL;		/* read error */
    }
    endbuf = rbuf + tot;			/* count space for pointers */
    align = (sizeof(char*)-1) - ((tot + sizeof(char*)-1) & (sizeof(char*)-1));
    for (c = 0, p = rbuf; p < endbuf; p++)
    	if (!*p)
	    c += sizeof(char*);
    c += sizeof(char*);				/* one extra for NULL term */

    rbuf = xrealloc(rbuf, tot + c + align);	/* make room for ptrs AT END */
    endbuf = rbuf + tot;			/* addr just past data buf */
    q = ret = (char**) (endbuf+align);		/* ==> free(*ret) to dealloc */
    *q++ = p = rbuf;				/* point ptrs to the strings */
    endbuf--;					/* do not traverse final NUL */
    while (++p < endbuf) 
    	if (!*p)				/* NUL char implies that */
	    *q++ = p+1;				/* next string -> next char */

    *q = 0;					/* null ptr list terminator */
    return ret;
}

int file2str(char *directory, char *what, char *ret, int cap) {
	static char filename[80];
	int fd, num_read;

	sprintf(filename, "%s/%s", directory, what);
	if((fd = open(filename, O_RDONLY, 0)) == -1 ) 
		return -1;
	if((num_read = read(fd, ret, cap - 1)) <= 0 ) 
		return -1;
	ret[num_read] = 0;
	close(fd);
	return num_read;
}

char *print_strlist(char **strs, char* sep, int max) {
	char *line, buf[64] = "";
	int i, n, seplen = strlen(sep);
	
	line = (char *)malloc(max * sizeof(char));
	for(n=0; *strs && n < max; strs++){
		for(i=0; strs[0][i] && n+i < max; i++)
	    	if(isprint(strs[0][i]) || strs[0][i] == ' ') {
				line[strlen(line)] = strs[0][i];
				line[strlen(line)] = '\0';
	    	} else {
				if(max-(n+i) > 3) {
		    		sprintf(buf, "\\%03o", strs[0][i]);
		    		strcat(line, buf);
		    		n += 3; /* 4 printed, but i counts one */
				}
	    	}
			n += i;
			if(n + seplen < max) {
		    	sprintf(buf, "%s", sep);
		    	strcat(line, buf);
	   		n += seplen;
			}
			
	}
	return line;
}

// ----------------- DEV / TTYS ---------------

#define __KERNEL__
#include <linux/kdev_t.h>
#undef __KERNEL__

#define DEVDIR		"/dev"
#define DEVTAB		"psdevtab"
static char *devtab_paths[] = {
    "/etc/" DEVTAB,
    "/tmp/" DEVTAB,
    "%s/."  DEVTAB,
    NULL
};
#define DEVINITIALMODE	0664
#define DEV_MAX_PATH	(5+256)
#define DEV_NAME_MAX	8

static dev_t major[] = { 2, 3, 4, 5, 19, 20 };
#define Nminor		256
#define Nmajor		(sizeof(major)/sizeof(dev_t))
#define Ndev		(Nmajor*Nminor)
#define Ndevtab		(Ndev*DEV_NAME_MAX)

static char* devtab;	/* the memory buffer holding all the name strings */

/* This macro evaluates to the address into the table of the string of
   DEV_NAME_MAX chars for the device with major m, minor n. */
#define TAB(m,n) (devtab + (m)*(Nminor*DEV_NAME_MAX) + (n)*DEV_NAME_MAX)

static int devtab_initialized = 0;

static char* name_to_path(char* name);	/* forward declarations */
static int   init_devtab (void);

/* Device Name -> Number Map
   many-to-one: -1 on failed match.
*/
dev_t name_to_dev(char* name) {
    static struct stat sbuf;
    return (stat(name_to_path(name), &sbuf) < 0) ? -1 : sbuf.st_rdev;
}

/* find m in a[] assuming a is sorted into ascending order */
/* in-line linear search placeholder until more majors warrant binary search */
static __inline__ int lookup(dev_t m, dev_t* a, int n) {
    int k;
    for(k=0; k < n && a[k] != m; k++)
	;
    return (k < n) ? k : -1;
}

/* Device Number -> Name Map
   one-to-many: first directory order match in DEVDIR, "" on failed match.
*/
char* dev_to_name(dev_t num) {
    static char	rval[DEV_NAME_MAX+1];
    dev_t m = MAJOR(num), n = MINOR(num), tmp;
    if (!devtab_initialized && !init_devtab())
	return "";
    if ((tmp = lookup(m, major, Nmajor)) == (dev_t)-1)
	return "";
    strncpy(rval, TAB(tmp,n), DEV_NAME_MAX);
    rval[DEV_NAME_MAX] = '\0';
    return rval;
}

static int init_devtab() {
    static struct stat sbuf;
    static int fd;
    char **fmt, path[64], *HOME = getenv("HOME") ? getenv("HOME") : "";
    for (fmt = devtab_paths; *fmt; fmt++) {
	snprintf(path, sizeof path, *fmt, HOME);
	if ( (fd = open(path, O_RDONLY)) < 0	/* open DEVTAB file */
	     || fstat(fd, &sbuf) < 0		/* fstat it */
	     || sbuf.st_size != Ndevtab		/* make sure it's the right size */
	     || (devtab = mmap(0, Ndevtab, PROT_READ, MAP_SHARED, fd, 0)) ==
	     (caddr_t) -1
	     || close(fd) == -1)
	    {	/* could not open for read, attempt to fix/create */
		int oumsk = umask(0);
		if (devtab)
		    munmap(devtab, Ndevtab);
		if (((fd = open(path, O_RDWR|O_TRUNC|O_CREAT, DEVINITIALMODE)) == -1 &&
		     (unlink(path), fd = open(path, O_RDWR|O_TRUNC|O_CREAT,
		     DEVINITIALMODE)) == -1)
		     || !dev_to_devtab(fd)) {
		    close(fd);          /* either both opens failed or the constructor
		    failed */
		    unlink(path);	/* in case we created but could not fill a file */
		    umask(oumsk);
		    continue;
		} else {
		    devtab_initialized = 1;
		    close(fd);
		    umask(oumsk);
		    return 1;
		}
	    }
	else
	    return devtab_initialized = 1;
    }
    return devtab_initialized;
}

/* stat every file in DEVDIR saving its basename in devtab[] if it has
   a MAJOR(st_rdev) in our list of majors. return 0 on error otherwise 1. */
static int dev_to_devtab(int fd) {
    static struct stat sbuf;
    int i;
    dev_t m;
    struct dirent* ent;
    DIR* dev;
    
    if (!(dev = opendir(DEVDIR))) {
	fprintf(stderr, "%s: %s\nCannot generate device number -> name mapping.\n",
		DEVDIR, sys_errlist[errno]);
	return 0;
    }
    if (!(devtab = malloc(Ndevtab))) {
	fprintf(stderr, "%s: could not allocate memory\n", sys_errlist[errno]);
	return 0;
    }
    memset((void*)devtab, 0, Ndevtab);
    while ((ent = readdir(dev))) {	/* loop over all dirents in DEVDIR */
	if (lstat(name_to_path(ent->d_name), &sbuf) < 0
	    || !S_ISCHR(sbuf.st_mode))	    /* only look for char special devs */
	    continue;			    /* due to overloading of majors */
	m = MAJOR(sbuf.st_rdev);	    /* copy name to appropriate spot */
	if ((i = lookup(m, major, Nmajor)) != -1)
	    strncpy(TAB(i,MINOR(sbuf.st_rdev)), ent->d_name, DEV_NAME_MAX);
    }
    closedir(dev);
    if (write(fd, devtab, Ndevtab) != Ndevtab) /* probably no disk space */
	return 0;
    return 1;
}

static char path[DEV_MAX_PATH];

static char* name_to_path(char* name) {
    static char* Path;
    if (!Path) {
	strcpy(path, DEVDIR);		/* copy DEVDIR */
	path[sizeof(DEVDIR) - 1] = '/';	/* change NUL to '/' */
	Path = path + sizeof(DEVDIR);	/* put Path at start of basename */
    }
    strncpy(Path, name, DEV_MAX_PATH - sizeof(DEVDIR));
    return path;
}

/* Re-implementation of old interface with the new generic functions. */

/* This does exactly the same thing as name_to_dev only now a full "ttyXX"
   specification will work as well.
*/
int tty_to_dev(char *tty) {
    static char pref_name_1[32] = "tty", *pnam1 = pref_name_1 + 3,
	        pref_name_2[32] = "cu",  *pnam2 = pref_name_2 + 2;
    dev_t num;
    if ((num = name_to_dev(tty)) != (dev_t) -1)	/* try tty straight up */
	return num;
    strncpy(pnam1, tty, 32 - 3);	/* try with "tty" prepended */
    if ((num = name_to_dev(pref_name_1)) != (dev_t) -1)
	return num;
    strncpy(pnam2, tty, 32 - 2);	/* try with "cu" prepended */
    if ((num = name_to_dev(pref_name_2)) != (dev_t) -1)
	return num;
    return -1;		/* no match */
}

/* new abstraction that can maybe be generalized a little better. */
char* abbrev_of_tty(char *tty) {
    static char temp[32];		/* return buf: good only until next call */
    char *pos = strpbrk(tty, "yu");	/* end of (presumed) prefices: tty*, cu* */
    temp[0] = 0;
    if (tty && tty[0] && pos && pos[0] && pos[1])
	sprintf(temp, "%*.*s", 3, 3, pos + 1);
    else
	strncpy(temp, " ? ", 31);
    return temp;
}

/* Do in-place modification of the 4-buffer `tty' based upon `dev' */
void dev_to_tty(char *tty, int dev) {
    char* newc = abbrev_of_tty(dev_to_name(dev));
    strncpy(tty, newc, 4);
}


// ---------------- USER NAME --------------------------
#define	HASHSIZE	16			/* power of 2 */
#define	HASH(x)		((x) & (HASHSIZE - 1))

static struct pwbuf {
	int uid;
	char name[12];
	struct pwbuf *next;
} *pwhash[HASHSIZE];

char *user_from_uid(int uid)
{
	struct pwbuf **p;
	struct passwd *pw;

	p = &pwhash[HASH(uid)];
	while(*p) {
		if((*p)->uid == uid)
	    	return((*p)->name);
		p = &(*p)->next;
	}
	*p = (struct pwbuf *)xmalloc(sizeof(struct pwbuf));
	(*p)->uid = uid;
	if((pw = getpwuid(uid)) == NULL)
		sprintf((*p)->name, "#%d", uid);
	else
		sprintf((*p)->name, "%-.8s", pw->pw_name);
	(*p)->next = NULL;
	return((*p)->name);
}

void bad_user_access_length() { }


// ------------------ MALLOC -------------------
void *xcalloc(void *pointer, int size) {
    void * ret;
    if (pointer)
        free(pointer);
    if (!(ret = calloc(1, size))) {
        fprintf(stderr, "xcalloc: allocation error, size = %d\n", size);
        exit(1);
    } else {
        return ret;
    }
}

void *xmalloc(unsigned int size) {
    void *p;

    if (size == 0)
        ++size;
    p = malloc(size);
    if (!p) {
	fprintf(stderr, "xmalloc: malloc(%d) failed", size);
	perror(NULL);
	exit(1);
    }
    return(p);
}

void *xrealloc(void *oldp, unsigned int size) {
    void *p;

    if (size == 0)
        ++size;
    p = realloc(oldp, size);
    if (!p) {
	fprintf(stderr, "xrealloc: realloc(%d) failed", size);
	perror(NULL);
	exit(1);
    }
    return(p);
}

NSRect icnRect = {{0, 0}, {15, 15}};
NSRect nameRect = {{20, 0}, {60, 15}};
NSRect statRect = {{80, 0}, {15, 15}};
NSRect usrIcnRect = {{100, 0}, {15, 15}};
NSRect usrRect = {{120, 0}, {50, 15}};
NSRect ttyIcnRect = {{180, 0}, {15, 15}};
NSRect ttyRect = {{200, 0}, {30, 15}};
NSRect cmdIcnRect = {{235, 0}, {15, 15}};
NSRect commRect = {{255, 0}, {300, 15}};

@implementation ProcObj

- (id)initInProcsFrame:(ProcsFrame *)parent
			  	  withPath:(NSString *)apath
{
	NSFont *font;
	NSString *cellTxt;
	char *buffp;
	NSRect myRect = {{0, 0}, {400, 15}};

	self = [super init];
  	if(self) {
		[self setFrame: myRect];
		font = [NSFont systemFontOfSize: 10];
		myFrame = parent;
		
		isSelect = NO;
		intree = NO;
		xShift = 0;
		nchilds = 0;
		parentid = 0;
		type = 0;
		isTty = NO;
		isCmd = NO;
		explWin = nil;
		
		path = [[NSString alloc] initWithString: apath];
		[self getProcData];
		name = [[NSString alloc] initWithCString: pinf1.cmd];
		
		icnCell = [[NSCell alloc] initImageCell: [myFrame procIcon]];

		cellTxt = [NSString stringWithCString: pinf1.cmd];
		nameCell = [[NSCell alloc] initTextCell: cellTxt];
		[nameCell setFont: font];
		[nameCell setAlignment: NSLeftTextAlignment];

		cellTxt = [NSString stringWithFormat:@"%c", pinf1.state];
		statCell = [[NSCell alloc] initTextCell: cellTxt];
		[statCell setFont: font];
		[statCell setAlignment: NSLeftTextAlignment];

		usrIcnCell = [[NSCell alloc] initImageCell: [myFrame usrIcn]];

		cellTxt = [NSString stringWithCString: user];
		usrCell = [[NSCell alloc] initTextCell: cellTxt];
		[usrCell setFont: font];
		[usrCell setAlignment: NSLeftTextAlignment];

		if(isTty) {
			ttyIcnCell = [[NSCell alloc] initImageCell: [myFrame ttyIcn]];
			cellTxt = [NSString stringWithCString: ttyc];
			ttyCell = [[NSCell alloc] initTextCell: cellTxt];
			[ttyCell setFont: font];
			[ttyCell setAlignment: NSLeftTextAlignment];
		}

		if(isCmd) {
			cmdIcnCell = [[NSCell alloc] initImageCell: [myFrame cmdIcn]];
			buffp = print_strlist(cmdline, " ", 90);
			cellTxt = [NSString stringWithCString: buffp];
			free(buffp);
			commCell = [[NSCell alloc] initTextCell: cellTxt];
			[commCell setFont: font];
			[commCell setAlignment: NSLeftTextAlignment];
		}

		[self Activate];
	}
	return self;
}

- (void) dealloc
{
	[icnCell release];
	[usrIcnCell release];
	[ttyIcnCell release];
	[cmdIcnCell release];
	[nameCell release];
	[usrCell release];
	[statCell release];
	[ttyCell release];
	[commCell release];
	[path release];
	[name release];
	if(explWin) {
		[explWin release];
		[win release];
	}
  	[super dealloc];
}

- (void)getProcData
{
	struct stat listStat;
	char sbuf[256] = "";

	stat([path cString], &listStat);
	uid = listStat.st_uid;
	file2str((char *)[path cString], "stat", sbuf, sizeof(sbuf));
	pinf1 = stat2proc(sbuf);
	dev_to_tty(ttyc, pinf1.tty);
	if(strcmp(ttyc, " ? ") != 0)
		isTty = YES;
	strncpy(user, user_from_uid(uid), sizeof(user));
	file2str((char *)[path cString], "statm", sbuf, sizeof(sbuf));
	pinf2 = statm2proc(sbuf);	
	cmdline = file2strvec((char *)[path cString], "cmdline");
	if(cmdline != NULL)
		isCmd = YES;	
}

- (void)addChildId:(int)anid
{
	childsid[nchilds] = anid;
	nchilds++;
}

- (void)resetNchilds
{
	nchilds = 0;
}

- (void)setId:(int)anid
{
	myId = anid;
}

- (void)setParentid:(int)anid
{
	parentid = anid;
}

- (void)setXShift:(int)shift
{
	xShift = shift;
}

- (void)setInTree
{
	intree = YES;
}

- (int)pid
{
	return pinf1.pid;
}

- (int)ppid
{
	return pinf1.ppid;
}

- (NSString *)ppidname
{
	if(pinf1.pid != 1)
		return [myFrame nameOfProcWithPid: pinf1.ppid];
	return @"-";
}

- (NSString *)myName
{
	return name;
}

- (NSString *)user
{
	return [NSString stringWithCString: user];
}

- (NSString *)state
{
	return [NSString stringWithFormat:@"%c", pinf1.state];
}

- (NSString *)tty
{
	return [NSString stringWithCString: ttyc];
}

- (int)parentid
{
	return parentid;
}

- (int)myId
{
	return myId;
}

- (int)xShift
{
	return xShift;
}

- (long)start_time
{
	return pinf1.start_time;
}

- (long)utime
{
	return pinf1.utime;
}

- (long)stime
{
	return pinf1.stime;
}

- (long)cutime
{
	return pinf1.cutime;
}

- (long)cstime
{
	return pinf1.cstime;
}

- (long)size
{
	return pinf2.size;
}

- (long)resident
{
	return pinf2.resident;
}

- (long)share
{
	return pinf2.share;
}

- (unsigned long)vsize
{
	return pinf1.vsize;
}

- (unsigned long)start_code
{
	return pinf1.start_code;
}

- (unsigned long)end_code
{
	return pinf1.end_code;
}

- (NSString *)cmdLine
{
	NSString *line;
	char buff[512] = "";
	
	sprintf(buff, "%s", print_strlist(cmdline, " ", 512));
	line = [NSString stringWithCString: buff];
	return line;
}

- (BOOL)intree
{
	return intree;
}

- (int)nchilds
{
	return nchilds;
}

- (int)childsid:(int)index
{
	return childsid[index];
}

- (BOOL)isSelect
{
	return isSelect;
}

- (void)Activate
{
	[icnCell setImage: [myFrame procIcon]];
	[nameCell setEnabled: YES];
	[statCell setEnabled: YES];
	[usrIcnCell setImage: [myFrame usrIcn]];
	[usrCell setEnabled: YES];
	if(isTty) {
		[ttyIcnCell setImage: [myFrame ttyIcn]];
		[ttyCell setEnabled: YES];
	}
	if(isCmd) {
		[cmdIcnCell setImage: [myFrame cmdIcn]];
		[commCell setEnabled: YES];
	}
	isselectable = YES; 
	[self setNeedsDisplay: YES];
}
								
- (void)DeActivate
{
	if(win) {
		[win close];
		[explWin release];
		win = nil;
		explWin = nil;
	}

	if(isSelect)
		[self Unselect];

	[icnCell setImage: [myFrame procIconUnsl]];
	[nameCell setEnabled: NO];
	[statCell setEnabled: NO];
	[usrIcnCell setImage: [myFrame usrIcnUnsl]];
	[usrCell setEnabled: NO];
	if(isTty) {
		[ttyIcnCell setImage: [myFrame ttyIcnUnsl]];
		[ttyCell setEnabled: NO];
	}
	if(isCmd) {
		[cmdIcnCell setImage: [myFrame cmdIcnUnsl]];
		[commCell setEnabled: NO];
	}
	isselectable = NO; 
	[self setNeedsDisplay: YES];
}	
 	
- (void)Select
{
	if(isselectable) {
		isSelect = YES;
		[myFrame unselectOthers: myId];
		[self setNeedsDisplay: YES];
	}
}

- (void)Unselect
{
	isSelect = NO;
	[self setNeedsDisplay: YES];
}

- (void)openExplWin
{
	NSRect winRect = {{200, 200}, {190, 340}};
	NSRect viewRect;
	unsigned int style = NSTitledWindowMask | NSMiniaturizableWindowMask;

	if(!isselectable)
		return;

	if(win) {
		[win orderFront:nil];
		return;
	}
	
	win = [[NSWindow alloc] initWithContentRect:winRect
												 styleMask:style
												   backing:NSBackingStoreRetained
													  defer:NO];
	[win setTitle:@"Process Inspector"];
	viewRect = NSMakeRect(0, 0, [win frame].size.width, [win frame].size.height);
	explWin = [[ProcObjWin alloc] initWithFrame:viewRect procObj:self];
	[[win contentView] addSubview: explWin];
	[win display];
	[win orderFront:nil];
}

- (void)closeExplWin:(id)sender
{
	[win close];
	[explWin release];
	win = nil;
	explWin = nil;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}

- (void)mouseDown:(NSEvent *)theEvent
{
	int count;
	
	count = [theEvent clickCount];
	if(count == 1)
		[self Select];
	else {
		[self openExplWin];
	}
	
	[super mouseDown: theEvent];
}

- (void)mouseUp:(NSEvent *)theEvent
{
	[super mouseUp: theEvent];
}

- (void) mouseMoved: (NSEvent *)theEvent
{
	[super mouseMoved: theEvent];
}

- (void)drawRect:(NSRect)rect
{ 
	if(isSelect)
  		[[NSColor whiteColor] set];
	else
  		[[NSColor lightGrayColor] set];
	NSRectFill(rect);
	
	[icnCell drawInteriorWithFrame: icnRect inView: self];
	[nameCell drawInteriorWithFrame: nameRect inView: self];
	[statCell drawInteriorWithFrame: statRect inView: self];
	[usrIcnCell drawInteriorWithFrame: usrIcnRect inView: self];
	[usrCell drawInteriorWithFrame: usrRect inView: self];
	if(isTty) {
		[ttyIcnCell drawInteriorWithFrame: ttyIcnRect inView: self];
		[ttyCell drawInteriorWithFrame: ttyRect inView: self];
	}
	if(isCmd) {
		[cmdIcnCell drawInteriorWithFrame: cmdIcnRect inView: self];
		[commCell drawInteriorWithFrame: commRect inView: self];
	}
}

@end
