#include "procsframe.h"
#include "procedit.h"

int compid(const void *a, const void *b)
{
	return(*((int *)a) - *((int *)b));	
}

@implementation ProcsFrame

- (id)initWithFrame:(NSRect)frameRect;
{
	NSString *iconpath;

	self = [super initWithFrame: frameRect];
  	if(self) {
		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/proc.tiff"];
		procIcon = [[NSImage alloc] initWithContentsOfFile: iconpath];
		
		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/proc_l.tiff"];
		procIconUnsl = [[NSImage alloc] initWithContentsOfFile: iconpath];
		
		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/user.tiff"];
		usrIcn = [[NSImage alloc] initWithContentsOfFile: iconpath];

		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/user_l.tiff"];
		usrIcnUnsl = [[NSImage alloc] initWithContentsOfFile: iconpath];

		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/monitor.tiff"];
		ttyIcn = [[NSImage alloc] initWithContentsOfFile: iconpath];

		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/monitor_l.tiff"];
		ttyIcnUnsl = [[NSImage alloc] initWithContentsOfFile: iconpath];

		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/pencil.tiff"];
		cmdIcn = [[NSImage alloc] initWithContentsOfFile: iconpath];
		
		iconpath = [[[NSBundle mainBundle] bundlePath]
		stringByAppendingString:@"/Resources/pencil_l.tiff"];
		cmdIcnUnsl = [[NSImage alloc] initWithContentsOfFile: iconpath];
		
		[self makeProcsObjects];
	}
	return self;
}

- (void)dealloc
{
	[list removeAllObjects];
	[list release];
	[procIcon release];
	[usrIcn release];
	[ttyIcn release];
	[cmdIcn release];
	[procIconUnsl release];
	[usrIcnUnsl release];
	[ttyIcnUnsl release];
	[cmdIcnUnsl release];
  	[super dealloc];
}

- (void)makeProcsObjects
{
	struct dirent **namelist;
	struct stat listStat;
	int i, j, n, m;	
	BOOL found;
	ProcObj *pobj, *pobj1, *pobj2;
	NSMutableArray *usersArr;
	NSString *user;
	char path[64] = "";

	templist1 = [[NSMutableArray alloc] initWithCapacity: 1];
	m = scandir("/proc", &namelist, 0, alphasort);
	for(n = 0; n < m; n++) {
		if(*namelist[n]->d_name > '0' && *namelist[n]->d_name <= '9') {
			sprintf(path, "/proc/%s", namelist[n]->d_name);
			if(stat(path, &listStat) != -1) {
				pobj = [[ProcObj alloc] initInProcsFrame: self
			  	  			withPath: [NSString stringWithCString: path]];
				[templist1 addObject: pobj];
				[pobj release];
			}
		}
	}

	for(i = 0; i < [templist1 count]; i++)
		procspids[i] = [[templist1 objectAtIndex: i] pid];
	qsort(procspids, [templist1 count], sizeof(int), compid);

	templist2 = [[NSMutableArray alloc] initWithCapacity: 1];
	for(i = 0; i < [templist1 count]; i++) {
		for(j = 0; j < [templist1 count]; j++) {
			if(procspids[i] == [[templist1 objectAtIndex: j] pid]) {
				[templist2 addObject: [templist1 objectAtIndex: j]];
			}
		}
	}

	list = [[NSMutableArray alloc] initWithCapacity: 1];
	for(i = 0; i < [templist2 count]; i++) {
		[list addObject: [templist2 objectAtIndex: i]];
		[[list objectAtIndex: i] setId: i];
	}
	[templist1 release];
	[templist2 release];

	for(i = 0; i < [list count]; i++) {
		pobj1 = [list objectAtIndex: i];
		for(j = 0; j < [list count]; j++) {
			pobj2 = [list objectAtIndex: j];
			if([pobj2 ppid] == [pobj1 pid]) {
				[pobj2 setParentid: i];
				[pobj2 setXShift: [pobj1 xShift]+1];
				[pobj1 addChildId: j];
			}
		}
	}

	templist1 = [[NSMutableArray alloc] initWithCapacity: 1];
	for(i = 0; i < [list count]; i++)
		[self makeProcTree: [list objectAtIndex: i]];
		
	[list release];
	list = [[NSMutableArray alloc] initWithCapacity: 1];
	for(i = 0; i < [templist1 count]; i++) 
		[list addObject: [templist1 objectAtIndex: i]];
	[templist1 release];

	for(i = 0; i < [list count]; i++) {
		pobj1 = [list objectAtIndex: i];
		[pobj1 resetNchilds];
		for(j = 0; j < [list count]; j++) {
			pobj2 = [list objectAtIndex: j];
			if([pobj2 ppid] == [pobj1 pid]) {
				[pobj2 setParentid: i];
				[pobj2 setXShift: [pobj1 xShift]+1];
				[pobj1 addChildId: j];
			}
		}
	}

	usersArr = [NSMutableArray arrayWithCapacity: 1];
	for(i = 0; i < [list count]; i++) {
		found = NO;
		pobj = [list objectAtIndex: i];
		user = [pobj user];
		for(j = 0; j < [usersArr count]; j++) {
			if([user isEqualToString: [usersArr objectAtIndex: j]]) {
				found = YES;
				break;
			}
		}
		if(!found)
			[usersArr addObject: user];
	}
	[[ProcEdit sharedprocedit] updatePopUp: usersArr];
	
	for(i = 0; i < [list count]; i++) 
		[self addSubview: [list objectAtIndex:i]];
}

- (void)makeProcTree:(ProcObj *)insObj
{
	int i;
	
	if(![insObj intree]) {
		[insObj setInTree];
		[templist1 addObject: insObj];
		if([insObj nchilds] > 0) {
			for(i = 0; i < [insObj nchilds]; i++) {
				[self makeProcTree: [list objectAtIndex: [insObj childsid: i]]];
			}
		}
	}	
}

- (void)showProcsOfUser:(NSString *)usrname
{
	ProcObj *pobj;
	int i;
	
	if([usrname isEqualToString:@"root"]) {
		for(i = 0; i < [list count]; i++) {
			pobj = [list objectAtIndex: i];
			[pobj Activate];
		}
	} else {
		for(i = 0; i < [list count]; i++) {
			pobj = [list objectAtIndex: i];
			if([[pobj user] isEqualToString: usrname])
				[pobj Activate];
			else
				[pobj DeActivate];
		}
	}
	
	for(i = 0; i < [list count]; i++) {
		pobj = [list objectAtIndex: i];
		[pobj setNeedsDisplayInRect: [pobj frame]];
	}
	[self setNeedsDisplayInRect: [self frame]];
}

- (NSString *)nameOfProcWithPid:(int)procpid
{
	ProcObj *pobj;
	int i;

	for(i = 0; i < [list count]; i++) {
		pobj = [list objectAtIndex: i];
		if([pobj pid] == procpid)
			return [pobj myName];
	}
	return nil;
}

- (void)resizeWithOldSuperviewSize:(NSSize)oldFrameSize
{
	ProcObj *pobj;
	int i, vpos, newheight;
	NSRect newFrame, newObjFrame;
	
	newheight = 0;
	for(i = 0; i < [list count]; i++)
		newheight = newheight + 20;

	vpos = newheight;
	for(i = 0; i < [list count]; i++) {
		vpos = vpos - 20;
		pobj = [list objectAtIndex: i];
		newObjFrame = NSMakeRect([pobj xShift] * 20 + 5, vpos, 
									[self frame].size.width, 15);
		[pobj setFrame:newObjFrame];
      [pobj setNeedsDisplay:YES];
	}
	
	if(newheight > [self frame].size.height) {
		newFrame = NSMakeRect(0, [self frame].size.height - newheight, 
														[self frame].size.width, newheight);
		[self setFrame: newFrame];
	}
	[self setNeedsDisplay: YES];
}

- (void)unselectOthers:(int)objid
{
	ProcObj *pobj;
	int i;
	
	for(i = 0; i < [list count]; i++) {
		pobj = [list objectAtIndex: i];
		if([pobj myId] != objid)
			[pobj Unselect];
	}
}

- (NSImage *)procIcon
{
	return procIcon;
}

- (NSImage *)usrIcn
{
	return usrIcn;
}

- (NSImage *)ttyIcn
{
	return ttyIcn;
}

- (NSImage *)cmdIcn
{
	return cmdIcn;
}

- (NSImage *)procIconUnsl
{
	return procIconUnsl;
}

- (NSImage *)usrIcnUnsl
{
	return usrIcnUnsl;
}

- (NSImage *)ttyIcnUnsl
{
	return ttyIcnUnsl;
}

- (NSImage *)cmdIcnUnsl
{
	return cmdIcnUnsl;
}

- (void)drawRect:(NSRect)rect
{
	ProcObj *pobj;
	NSRect r, cr;
	int i;

  	[[NSColor blackColor] set];
	
	for(i = 0; i < [list count]; i++) {
		pobj = [list objectAtIndex: i];
		
		if([pobj ppid] != 0) {
			r = [pobj frame];
  			PSmoveto(r.origin.x -12, r.origin.y +7);
			PSlineto(r.origin.x -4, r.origin.y +7);
			PSstroke();
		}
		
		if([pobj nchilds] > 0) {
			r = [pobj frame];
			cr = [[list objectAtIndex: [pobj childsid: [pobj nchilds] -1]] frame];
  			PSmoveto(r.origin.x +7, r.origin.y -2);
			PSlineto(r.origin.x +7, cr.origin.y +7);
			PSstroke();
		}
	}	
} 

@end
